/***************************************************************************
 **
 **    This file defines the board specific definition
 **
 **    Used with ARM IAR C/C++ Compiler and Assembler.
 **
 **    (c) Copyright IAR Systems 2006
 **
 **    $Revision: 1.1.2.1 $
 **
 ***************************************************************************/
#ifndef __BOARD_H
#define __BOARD_H

// OSCs [Hz]
#define MAIN_OSC_FREQ	        (12000000UL)
#define LF_OSC_FREQ		(32768UL)

#define SYS_FREQ  48

// PLL
#if SYS_FREQ == 12
#define PLL_MUL               1
#define PLL_DIV               1
#elif SYS_FREQ == 24
#define PLL_MUL               2
#define PLL_DIV               1
#elif SYS_FREQ == 36
#define PLL_MUL               3
#define PLL_DIV               1
#elif SYS_FREQ == 48
#define PLL_MUL               4
#define PLL_DIV               1
#elif SYS_FREQ == 54
#define PLL_MUL               9
#define PLL_DIV               2
#elif SYS_FREQ == 60
#define PLL_MUL               5
#define PLL_DIV               1
#else
#error Unsupported system frequency
#endif

// SDRAM K4S561632L-TC75
// [ns]
#define SDRAM_BASE_ADDR       0x30000000

#if SYS_FREQ == 12
#define SDRAM_PERIOD          83    // 12MHz
#elif SYS_FREQ == 24
#define SDRAM_PERIOD          41.6  // 24MHz
#elif SYS_FREQ == 36
#define SDRAM_PERIOD          27    // 36MHz
#elif SYS_FREQ == 48
#define SDRAM_PERIOD          21    // 48MHz
#elif SYS_FREQ == 54
#define SDRAM_PERIOD          18.5  // 54MHz
#elif SYS_FREQ == 60
#define SDRAM_PERIOD          16.6  // 60MHz
#endif


#define P2C(Period)           (((Period<SDRAM_PERIOD)?0:(Int32U)((Flo32)Period/SDRAM_PERIOD))+1)

#define SDRAM_REFRESH         15625
#define SDRAM_TRP             20
#define SDRAM_TRAS            45
#define SDRAM_TAPR            1
#define SDRAM_TDAL            2
#define SDRAM_TWR             3
#define SDRAM_TRC             65
#define SDRAM_TRFC            66
#define SDRAM_TXSR            67
#define SDRAM_TRRD            15
#define SDRAM_TMRD            3

#define FLASH_TAC             120

#endif /* __BOARD_H */
