#include "sdram_lpc.h"
#include <board.h>
#include <stdio.h>
#include <iolpc2888.h>
#include "arm_comm.h"


#pragma location="ICODE"
static void Dly_us (Int32U Dly) {

  INT_REQ5 = 0x04010000;          // enable interrupt
  T0CTRL = 0;                     // stop counting
  T0LOAD = 12 * Dly;              // load period
  T0CLR = 0;                      // clear timer pendig interrupt
  T0CTRL_bit.TENAB = 1;           // enable counting
  while(!INT_PENDING_bit.TIMER0); // while
  T0CTRL_bit.TENAB = 0;           // stop counting

}


void InitSDRAMCtrl(void) {

  // assigne GPIO to EMC
  MODE0_0 = 0xFFFFFFFF;
  MODE0_1 = 0x000FFFFF;
  MODE1_0 = 0x00000000;
  MODE1_1 = 0x00000000;
  // enable EMC clk
  // disable fractional divider
  EMCESR0_bit.ESR_EN = EMCESR1_bit.ESR_EN = 0;
  // release resert of the EMC
  EMCRES = 1;
  // Init SDRAM controller and memory
  EMCCONTROL = 1; // Enable EMC
  EMCDYNAMICREADCONFIG = 1; // Command delayed strategy, using AHBHCLKDELAY
  EMCMISC = 0;
  EMCDYNAMICRASCAS_bit.CAS = 2;
  EMCDYNAMICRASCAS_bit.RAS = 2;
  EMCDYNAMICRP = P2C(SDRAM_TRP);
  EMCDYNAMICRAS = P2C(SDRAM_TRAS);
  EMCDYNAMICSREX = P2C(SDRAM_TXSR);
  EMCDYNAMICAPR = SDRAM_TAPR;
  EMCDYNAMICDAL = SDRAM_TDAL+P2C(SDRAM_TRP);
  EMCDYNAMICWR = SDRAM_TWR;
  EMCDYNAMICRC = P2C(SDRAM_TRC);
  EMCDYNAMICRFC = P2C(SDRAM_TRFC);
  EMCDYNAMICXSR = P2C(SDRAM_TXSR);
  EMCDYNAMICRRD = P2C(SDRAM_TRRD);
  EMCDYNAMICMRD = SDRAM_TMRD;
  EMCDYNAMICCONFIG = 0x0000680;        // 13 row, 9 - col, SDRAM
  // JEDEC General SDRAM Initialization Sequence
  // DELAY to allow power and clocks to stabilize ~100 us
  // NOP
  EMCDYNAMICCONTROL = 0x4183;
  Dly_us(200);
  // PALL
  EMCDYNAMICCONTROL_bit.SDRAMINI = 2;
  EMCDYNAMICREFRESH = 1;
  for(volatile Int32U i= 128; i; --i); // > 128 clk
  EMCDYNAMICREFRESH = P2C(SDRAM_REFRESH) >> 4;
  // COMM
  EMCDYNAMICCONTROL_bit.SDRAMINI = 1;
  // Busrt 8, Sequental, CAS-2
  volatile unsigned long Dummy = *(volatile unsigned short *)(SDRAM_BASE_ADDR | (0x23UL << 12));
  // NORM
  EMCDYNAMICCONTROL = 0x4000;
  EMCDYNAMICCONFIG_bit.BUF_EN = 1;

}


__ramfunc void SDRAM_check(void ) {

  static pInt32U pData;
  static Int32U i, j;

  // printf("\nWriting to SDRAM!!!\nProgress:");
  // for(i = 0, pData = (pInt32U)SDRAM_BASE_ADDR; i < 0x1000000/4; ++i)
  // for(i=0, j=0, pData = (pInt32U)SDRAM_BASE_ADDR; i < 0x2000000/4; ++i, ++j) {
  for(i=0, pData = (pInt32U)SDRAM_BASE_ADDR; i < 0x2000000/4; ++i) {

    //if(j>=0x20000) { printf("*"); j=0; }
    *pData++ = i;
  }

  // printf("\nVerifing to SDRAM!!!\nProgress:");
  // for(i = 0, pData = (pInt32U)SDRAM_BASE_ADDR; i < 0x1000000/4; ++i)
  // for(i=0, j=0, pData = (pInt32U)SDRAM_BASE_ADDR; i < 0x2000000/4; ++i, ++j) {
  for(i=0, pData = (pInt32U)SDRAM_BASE_ADDR; i < 0x2000000/4; ++i) {

    //if(j>=0x20000) { printf("*"); j=0; }
    if(*pData++ != i)
    {
      //printf("\nSDRAM test ERROR!!!");
      while(1);
    }
  }
}


