#include "test.h"
#include <iolpc2888.h>

// EXTENSION TEST
const unsigned long mask_port_p0     = 0x00000000;
const unsigned long mask_port_p1     = 0x000030C0;
const unsigned long mask_port_p2     = 0x0000000C;
const unsigned long mask_port_p3     = 0x0000006F;
const unsigned long mask_port_p4     = 0x00000FFF;
const unsigned long mask_port_p5     = 0x0000003F;
const unsigned long mask_port_p6     = 0x0000000F;

// variable for test
char state_p0_pin      = 90;
char state_p0_vcc      = 0;
char state_p0_gnd      = 0;

char state_p1_pin      = 90;
char state_p1_vcc      = 0;
char state_p1_gnd      = 0;

char state_p2_pin      = 90;
char state_p2_vcc      = 0;
char state_p2_gnd      = 0;

char state_p3_pin      = 90;
char state_p3_vcc      = 0;
char state_p3_gnd      = 0;

char state_p4_pin      = 90;
char state_p4_vcc      = 0;
char state_p4_gnd      = 0;

char state_p5_pin      = 90;
char state_p5_vcc      = 0;
char state_p5_gnd      = 0;

char state_p6_pin      = 90;
char state_p6_vcc      = 0;
char state_p6_gnd      = 0;

char j;

unsigned int ADCValue;

void Delay(unsigned long delay) {

  volatile unsigned long d;

  d = delay;

  while(d--);

}

void PullUpHigh(void) {

  // Set pull up to HIGH
  MODE0_2_bit.P2_1 = 1;
  MODE1_2_bit.P2_1 = 1;

  Delay(100);
}


void PullUpLow(void) {

  // Set pull up to LOW
  MODE0_2_bit.P2_1 = 0;
  MODE1_2_bit.P2_1 = 1;

  Delay(100);
}

// ADC TEST

#define CH_AD0    1
#define CH_AD1    2
#define CH_AD2    3
#define CH_AD3    4
#define CH_AD4    5
#define CH_DCD    6

void InitADC(void) {

  // Power Up
  ADCPD   = 0x0;

  // Enable ADC
  ADCCON  = 0x3;

}

unsigned int GetADCValue(unsigned char chanel) {


  // Select desired chanel
  if(chanel == CH_AD0)
    ADCSEL = 0xA;
  else if(chanel == CH_AD1)
    ADCSEL = (0xA)<<4;
  else if(chanel == CH_AD2)
    ADCSEL = (0xA)<<8;
  else if(chanel == CH_AD3)
    ADCSEL = (0xA)<<12;
  else if(chanel == CH_AD4)
    ADCSEL = (0xA)<<16;
  else if(chanel == CH_DCD)
    ADCSEL = (0xA)<<20;

  // ENABLE and START -> 1, CSCAN -> 0
  ADCCON = 0xB;

  // ENABLE -> 1, CSCAN and START -> 0
  ADCCON = 0x3;

  // Wait until conversion is in progress
  while(ADCCON&0x10);

  // Return converted data
  if(chanel == CH_AD0)
    return ADCR0;
  else if(chanel == CH_AD1)
    return ADCR1;
  else if(chanel == CH_AD2)
    return ADCR2;
  else if(chanel == CH_AD3)
    return ADCR3;
  else if(chanel == CH_AD4)
    return ADCR4;
  else if(chanel == CH_DCD)
    return ADCR5;

  return 0;
}


void Stop_(void) {

  // printf("\n\rTEST EXTENSION ERROR!!!!");
  while(1);
}

void TestExt(void) {

  // All port as Input
  // Port1
  MODE0_1 = 0x0;
  MODE1_1 = 0x0;
  // Port2
  MODE0_2 = 0x0;
  MODE1_2 = 0x0;
  // Port3
  MODE0_3 = 0x0;
  MODE1_3 = 0x0;
  // Port4
  MODE0_4 = 0x0;
  MODE1_4 = 0x0;
  // Port5
  MODE0_5 = 0x0;
  MODE1_5 = 0x0;
  // Port6
  MODE0_6 = 0x0;
  MODE1_6 = 0x0;


  // VCC Test ------------------------------------------------------------------
  PullUpHigh();

  // Check for zero at tested port
  if((PINS_0|(~mask_port_p0)) != 0xFFFFFFFF)  { state_p0_gnd = 1; Stop_(); }
  if((PINS_1|(~mask_port_p1)) != 0xFFFFFFFF)  { state_p1_gnd = 1; Stop_(); }
  if((PINS_2|(~mask_port_p2)) != 0xFFFFFFFF)  { state_p2_gnd = 1; Stop_(); }
  if((PINS_3|(~mask_port_p3)) != 0xFFFFFFFF)  { state_p3_gnd = 1; Stop_(); }
  if((PINS_4|(~mask_port_p4)) != 0xFFFFFFFF)  { state_p4_gnd = 1; Stop_(); }
  if((PINS_5|(~mask_port_p5)) != 0xFFFFFFFF)  { state_p5_gnd = 1; Stop_(); }
  if((PINS_6|(~mask_port_p6)) != 0xFFFFFFFF)  { state_p6_gnd = 1; Stop_(); }



  // VCC Test ------------------------------------------------------------------
  PullUpLow();

  // Check for high level at tested port
  if((PINS_0&mask_port_p0) != 0x0)  { state_p0_vcc = 1;  Stop_(); }
  if((PINS_1&mask_port_p1) != 0x0)  { state_p1_vcc = 1;  Stop_(); }
  if((PINS_2&mask_port_p2) != 0x0)  { state_p2_vcc = 1;  Stop_(); }
  if((PINS_3&mask_port_p3) != 0x0)  { state_p3_vcc = 1;  Stop_(); }
  if((PINS_4&mask_port_p4) != 0x0)  { state_p4_vcc = 1;  Stop_(); }
  if((PINS_5&mask_port_p5) != 0x0)  { state_p5_vcc = 1;  Stop_(); }
  if((PINS_6&mask_port_p6) != 0x0)  { state_p6_vcc = 1;  Stop_(); }


  // Running zero Port1 --------------------------------------------------------
  PullUpHigh();

    // loop for port 0
  for(j=0; j<32; j++) {

    // this port is not tested
    if(!((mask_port_p1)&(1<<j))) continue;

    MODE0_1 = (0);
    MODE1_1 = (1<<j);

    // just simple delay
    Delay(100);

    // check for other zero at port 0
    if((PINS_1|(~mask_port_p1)) != ((0xFFFFFFFF)&(~(1<<j))))  {  state_p1_pin = j;  Stop_();  }
    if((PINS_2|(~mask_port_p2)) != 0xFFFFFFFF)                {  state_p1_pin = j;  Stop_();  }
    if((PINS_3|(~mask_port_p3)) != 0xFFFFFFFF)                {  state_p1_pin = j;  Stop_();  }
    if((PINS_4|(~mask_port_p4)) != 0xFFFFFFFF)                {  state_p1_pin = j;  Stop_();  }
    if((PINS_5|(~mask_port_p5)) != 0xFFFFFFFF)                {  state_p1_pin = j;  Stop_();  }
    if((PINS_6|(~mask_port_p6)) != 0xFFFFFFFF)                {  state_p1_pin = j;  Stop_();  }

  }

  MODE0_1 = (0);
  MODE1_1 = (0);

  // Running zero Port2 --------------------------------------------------------
  PullUpHigh();

    // loop for port 0
  for(j=0; j<32; j++) {

    // this port is not tested
    if(!((mask_port_p2)&(1<<j))) continue;

    MODE0_2 = (2);
    MODE1_2 = (1<<j)|2;

    // just simple delay
    Delay(100);

    // check for other zero at port 0
    if((PINS_1|(~mask_port_p1)) != 0xFFFFFFFF)                {  state_p2_pin = j;  Stop_();  }
    if((PINS_2|(~mask_port_p2)) != ((0xFFFFFFFF)&(~(1<<j))))  {  state_p2_pin = j;  Stop_();  }
    if((PINS_3|(~mask_port_p3)) != 0xFFFFFFFF)                {  state_p2_pin = j;  Stop_();  }
    if((PINS_4|(~mask_port_p4)) != 0xFFFFFFFF)                {  state_p2_pin = j;  Stop_();  }
    if((PINS_5|(~mask_port_p5)) != 0xFFFFFFFF)                {  state_p2_pin = j;  Stop_();  }
    if((PINS_6|(~mask_port_p6)) != 0xFFFFFFFF)                {  state_p2_pin = j;  Stop_();  }

  }

  MODE0_2 = (0);
  MODE1_2 = (0);


  // Running zero Port3 --------------------------------------------------------
  PullUpHigh();

    // loop for port 0
  for(j=0; j<32; j++) {

    // this port is not tested
    if(!((mask_port_p3)&(1<<j))) continue;

    MODE0_3 = (0);
    MODE1_3 = (1<<j);

    // just simple delay
    Delay(100);

    // check for other zero at port 0
    if((PINS_1|(~mask_port_p1)) != 0xFFFFFFFF)                {  state_p3_pin = j;  Stop_();  }
    if((PINS_2|(~mask_port_p2)) != 0xFFFFFFFF)                {  state_p3_pin = j;  Stop_();  }
    if((PINS_3|(~mask_port_p3)) != ((0xFFFFFFFF)&(~(1<<j))))  {  state_p3_pin = j;  Stop_();  }
    if((PINS_4|(~mask_port_p4)) != 0xFFFFFFFF)                {  state_p3_pin = j;  Stop_();  }
    if((PINS_5|(~mask_port_p5)) != 0xFFFFFFFF)                {  state_p3_pin = j;  Stop_();  }
    if((PINS_6|(~mask_port_p6)) != 0xFFFFFFFF)                {  state_p3_pin = j;  Stop_();  }

  }

  MODE0_3 = (0);
  MODE1_3 = (0);


  // Running zero Port4 --------------------------------------------------------
  PullUpHigh();

    // loop for port 0
  for(j=0; j<32; j++) {

    // this port is not tested
    if(!((mask_port_p4)&(1<<j))) continue;

    MODE0_4 = (0);
    MODE1_4 = (1<<j);

    // just simple delay
    Delay(100);

    // check for other zero at port 0
    if((PINS_1|(~mask_port_p1)) != 0xFFFFFFFF)                {  state_p4_pin = j;  Stop_();  }
    if((PINS_2|(~mask_port_p2)) != 0xFFFFFFFF)                {  state_p4_pin = j;  Stop_();  }
    if((PINS_3|(~mask_port_p3)) != 0xFFFFFFFF)                {  state_p4_pin = j;  Stop_();  }
    if((PINS_4|(~mask_port_p4)) != ((0xFFFFFFFF)&(~(1<<j))))  {  state_p4_pin = j;  Stop_();  }
    if((PINS_5|(~mask_port_p5)) != 0xFFFFFFFF)                {  state_p4_pin = j;  Stop_();  }
    if((PINS_6|(~mask_port_p6)) != 0xFFFFFFFF)                {  state_p4_pin = j;  Stop_();  }

  }


  MODE0_4 = (0);
  MODE1_4 = (0);


  // Running zero Port5 --------------------------------------------------------
  PullUpHigh();

    // loop for port 0
  for(j=0; j<32; j++) {

    // this port is not tested
    if(!((mask_port_p5)&(1<<j))) continue;

    MODE0_5 = (0);
    MODE1_5 = (1<<j);

    // just simple delay
    Delay(100);

    // check for other zero at port 0
    if((PINS_1|(~mask_port_p1)) != 0xFFFFFFFF)                {  state_p5_pin = j;  Stop_();  }
    if((PINS_2|(~mask_port_p2)) != 0xFFFFFFFF)                {  state_p5_pin = j;  Stop_();  }
    if((PINS_3|(~mask_port_p3)) != 0xFFFFFFFF)                {  state_p5_pin = j;  Stop_();  }
    if((PINS_4|(~mask_port_p4)) != 0xFFFFFFFF)                {  state_p5_pin = j;  Stop_();  }
    if((PINS_5|(~mask_port_p5)) != ((0xFFFFFFFF)&(~(1<<j))))  {  state_p5_pin = j;  Stop_();  }
    if((PINS_6|(~mask_port_p6)) != 0xFFFFFFFF)                {  state_p5_pin = j;  Stop_();  }

  }


  MODE0_5 = (0);
  MODE1_5 = (0);

  // Running zero Port6 --------------------------------------------------------
  PullUpHigh();

    // loop for port 0
  for(j=0; j<32; j++) {

    // this port is not tested
    if(!((mask_port_p6)&(1<<j))) continue;


    MODE0_6 = (0);
    MODE1_6 = (1<<j);

    // just simple delay
    Delay(100);

    // check for other zero at port 0
    if((PINS_1|(~mask_port_p1)) != 0xFFFFFFFF)                {  state_p6_pin = j;  Stop_();  }
    if((PINS_2|(~mask_port_p2)) != 0xFFFFFFFF)                {  state_p6_pin = j;  Stop_();  }
    if((PINS_3|(~mask_port_p3)) != 0xFFFFFFFF)                {  state_p6_pin = j;  Stop_();  }
    if((PINS_4|(~mask_port_p4)) != 0xFFFFFFFF)                {  state_p6_pin = j;  Stop_();  }
    if((PINS_5|(~mask_port_p5)) != 0xFFFFFFFF)                {  state_p6_pin = j;  Stop_();  }
    if((PINS_6|(~mask_port_p6)) != ((0xFFFFFFFF)&(~(1<<j))))  {  state_p6_pin = j;  Stop_();  }

  }


  MODE0_6 = (0);
  MODE1_6 = (0);
}

void TestAnalog(void) {

  InitADC();

  ADCValue = GetADCValue(CH_AD0);
  if((ADCValue<820)||(ADCValue>880)) { Stop_();  }
  ADCValue = GetADCValue(CH_AD1);
  if((ADCValue<640)||(ADCValue>720)) { Stop_();  }
  ADCValue = GetADCValue(CH_AD2);
  if((ADCValue<470)||(ADCValue>530)) { Stop_();  }
  ADCValue = GetADCValue(CH_AD3);
  if((ADCValue<300)||(ADCValue>380)) { Stop_();  }
  ADCValue = GetADCValue(CH_AD4);
  if((ADCValue<140)||(ADCValue>200)) { Stop_();  }

}
