#include "timers.h"
#include <iolpc2888.h>

#define TIMER0_TICK_PER_SEC   5
#define TIMER0_IN_FREQ        12000000

/*************************************************************************
 * Function Name: Timer0IntrHandler
 * Parameters: none
 *
 * Return: none
 *
 * Description: Timer 0 interrupt handler
 *
 *************************************************************************/
void Timer0IntrHandler (void)
{
  T0CLR = 0;              // clear timer interrupt
  MODE0_2_bit.P2_1 ^= 1;  // toogle P2.1
}




/*************************************************************************
 * Function Name: InitTimer0
 * Parameters: none
 *
 * Return: none
 *
 * Description: Initialize Timer0 interface
 *
 *************************************************************************/
void InitTimer0(void){

  // Init led D9 pin (P2.1)
  MODE1_2_bit.P2_1 = 1;
  MODE0_2_bit.P2_1 = 1;

  // Init Time0
  T0CTRL_bit.TENAB = 0;     // disable counting
  T0CTRL_bit.TMODE = 1;     // enable auto reload from T0LOAD register
  T0CTRL_bit.PRESCALE = 2;  // prescaler = CGU clock rate / 256
  // set timer 0 period
  T0LOAD = (TIMER0_IN_FREQ)/(256 * TIMER0_TICK_PER_SEC);
  T0CLR = 0;                // clear timer pendig interrupt
  // init timer 0 interrupt
  IrqTable[VIC_TIMER0].pIntrFunc    = Timer0IntrHandler;
  IrqTable[VIC_TIMER0].IntrPriority = 1;
  INT_REQ5 = (1UL << 29) |  // clear software interrupt
             (1UL << 28) |  // Update Priority
             (1UL << 27) |  // Update target
             (1UL << 26) |  // Update interrupt enable
             (1UL << 25) |  // Update interrupt active level
             (1UL << 16) |  // INTEN
             1;             // PRIO
  T0CTRL_bit.TENAB = 1;     // enable timier 0 counting
  __enable_interrupt();

}


